@@ -1,11 +1,13 @@
 from django.urls import path, include
-from django.contrib.auth.models import Group, User
-from rest_framework.decorators import api_view, permission_classes, authentication_classes
+from django.contrib.auth.models import User
+from rest_framework.decorators import api_view, authentication_classes
 from rest_framework.response import Response
+from django.utils.timezone import now
+from django.contrib.sessions.models import Session
 
 import requests
 from django.conf import settings
-from django.http import HttpResponseRedirect, HttpResponse
+from django.http import HttpResponseRedirect, HttpResponse, HttpResponseBadRequest
 
 
 def include_auth_urls():
@@ -14,6 +16,10 @@ def include_auth_urls():
         path(r'logout/', logout)
     ])
 
+def include_internal_urls():
+    return include([
+        path(r'sessions/flush/', flush_session_view),
+    ])
 
 @api_view(["GET"])
 @authentication_classes([])
@@ -80,3 +86,31 @@ def logout(request):
     response = HttpResponseRedirect(redirect_to=app_redirect_url)
     response.delete_cookie("sso_shared_session")
     return response
+
+@api_view(["POST"])
+def flush_session_view(request):
+    email = request.data["email"]
+    try:
+        user = User.objects.get(email=email)
+    except User.DoesNotExist:
+        return HttpResponseBadRequest({"reason": "no such user!"})
+    delete_all_unexpired_sessions_for_user(user)
+    return Response({"flushed_session": f"{email}"})
+
+
+def delete_all_unexpired_sessions_for_user(user):
+    """
+    This is pretty inefficient, and will log the user out across all devices
+    however, on our scale should work fine.
+    We can upgrade this setup later to store session/user pairs in a separate
+    custom table
+    :param user:
+    :return:
+    """
+    all_sessions = Session.objects.filter(expire_date__gte=now())
+    user_sessions = [
+        session.pk
+        for session in all_sessions
+        if str(user.pk) == session.get_decoded().get("_auth_user_id")
+    ]
+    Session.objects.filter(pk__in=user_sessions).delete()
